/*
 ESP8266 NRF24 Rules Server
 Version: 1.0
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 10/01/2018
 Last Modified: 10/01/2018
*/

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include "esp8266-sendemail/sendemail.h"

#include <SPI.h>
#include <EEPROM.h>
#include "mirf.h"
#include "nRF24L01.h"

const char *ssid = "";
const char *password = "";
IPAddress ip(192,168,1,3);  //Node static IP
IPAddress gateway(192,168,1,254);
IPAddress subnet(255,255,255,0);
ESP8266WebServer server ( 80 );

const char *emailFromName = "nRF-Server";
const char *emailFromAddress = "<nrfserver@yourdomain.com>";

const int led = 2;
uint8_t dataIn = 0;
uint8_t nrfVars[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; // Common vars which rules can read/write to
uint8_t nrfVarsTimeout[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; // If there is a time out when the value needs to be cleared

// Rules structure
#define MAX_RULES 30
struct Rule {
  uint8_t state;
  uint8_t dataMatch;
  uint8_t ruleAction;

  uint8_t ruleActionExtra1;
  uint8_t ruleActionExtra2;
  uint8_t ruleActionExtra3;
  
  char ruleActionExtra4[20]; // Email to or nRF TX
  char ruleActionExtra5[20]; // Email subject or nRF RX
};
Rule ruleList[MAX_RULES];
uint8_t ruleCount = 0;


void handleRoot() {
  digitalWrite ( led, 1 );

  String temp = "<html>\
<head>\
<title>nRF24 Rules</title>\
</head>\
<link rel='stylesheet' type='text/css' href='style.css'>\
<body>";

// POST: New rule
if (server.hasArg("bytematch")) {
  /*temp += "State: ";
  temp += server.arg("state");
  temp += "<br/>Hex match: ";
  temp += server.arg("bytematch");
  temp += "<br/>Action: ";
  temp += server.arg("action");
  temp += "<br/>Pin number: ";
  temp += server.arg("pin-number");
  temp += "<br/>Pin state: ";
  temp += server.arg("pin-state");
  temp += "<br/>Pin Time";
  temp += server.arg("pin-time");
  temp += "<br/>Variable number";
  temp += server.arg("variable-number");
  temp += "<br/>Variable byte";
  temp += server.arg("variable-byte");
  temp += "<br/>Variable timeout";
  temp += server.arg("variable-timeout");*/
  
  ruleList[ruleCount].state = server.arg("state").toInt();
  char byteConvertStr[10];
  strncpy(byteConvertStr, server.arg("bytematch").c_str(), 5);
  ruleList[ruleCount].dataMatch = strtoul(byteConvertStr, NULL, 16);
  ruleList[ruleCount].ruleAction = server.arg("action").toInt();
  
  if (ruleList[ruleCount].ruleAction == 0) { // Send email
    strncpy(ruleList[ruleCount].ruleActionExtra4, server.arg("email-to").c_str(), 20);
    strncpy(ruleList[ruleCount].ruleActionExtra5, server.arg("email-subject").c_str(), 20);
    
    /*temp += "email to ";
    temp += ruleList[ruleCount].ruleActionExtra4;
    temp += "<br/>";
    temp += "email subject ";
    temp += ruleList[ruleCount].ruleActionExtra5;
    temp += "<br/>";*/
  }
  else if (ruleList[ruleCount].ruleAction == 1) { // Pulse pin
    ruleList[ruleCount].ruleActionExtra1 = server.arg("pin-number").toInt();
    ruleList[ruleCount].ruleActionExtra2 = server.arg("pin-state").toInt();
    ruleList[ruleCount].ruleActionExtra3 = server.arg("pin-time").toInt();
    /*temp += "pin ";
    temp += ruleList[ruleCount].ruleActionExtra1;
    temp += "<br/>";
    temp += "state ";
    temp += ruleList[ruleCount].ruleActionExtra2;
    temp += "<br/>";
    temp += "time ";
    temp += ruleList[ruleCount].ruleActionExtra3;
    temp += "<br/>";*/
  }
  else if (ruleList[ruleCount].ruleAction == 2) { // nRF24 packet
    ruleList[ruleCount].ruleActionExtra1 = server.arg("nrf24-ch").toInt();
    ruleList[ruleCount].ruleActionExtra2 = server.arg("nrf24-byte").toInt();
    strncpy(ruleList[ruleCount].ruleActionExtra4, server.arg("nrf-tx").c_str(), 5);
    strncpy(ruleList[ruleCount].ruleActionExtra5, server.arg("nrf-rx").c_str(), 5);
    /*temp += "nrf ch ";
    temp += ruleList[ruleCount].ruleActionExtra1;
    temp += "<br/>";
    temp += "nrf byte ";
    temp += ruleList[ruleCount].ruleActionExtra2;
    temp += "<br/>";
    temp += "nrf tx ";
    temp += ruleList[ruleCount].ruleActionExtra4;
    temp += "<br/>";
    temp += "nrf rx ";
    temp += ruleList[ruleCount].ruleActionExtra5;
    temp += "<br/>";*/
  }
  else if (ruleList[ruleCount].ruleAction == 3) { // Write to variable
    ruleList[ruleCount].ruleActionExtra1 = server.arg("variable-number").toInt();
    ruleList[ruleCount].ruleActionExtra2 = server.arg("variable-byte").toInt();
    ruleList[ruleCount].ruleActionExtra3 = server.arg("variable-timeout").toInt();
    if (ruleList[ruleCount].ruleActionExtra3 == 0) { // No timeout set
      ruleList[ruleCount].ruleActionExtra3 = 254;
    }
  }
  else if (ruleList[ruleCount].ruleAction == 4) { // Reply with variable
    ruleList[ruleCount].ruleActionExtra1 = server.arg("variable-number").toInt();
  }
  
  ruleCount++;
}


// POST: Update / Delete rule
if (server.hasArg("submit-delete")) {
  uint8_t ruleDelete = server.arg("delete").toInt();

  if (ruleDelete >= 1) {
    ruleDelete--;
    ruleList[ruleDelete].state = 0;
    ruleList[ruleDelete].dataMatch = 0;
    ruleList[ruleDelete].ruleAction = 0;
    ruleList[ruleDelete].ruleActionExtra1 = 0;
    ruleList[ruleDelete].ruleActionExtra2 = 0;
    ruleList[ruleDelete].ruleActionExtra3 = 0;
    ruleList[ruleDelete].ruleActionExtra4[0] = 0;
    ruleList[ruleDelete].ruleActionExtra5[0] = 0;
  
    // Re-arrange rules
    for (uint8_t x = ruleDelete; x < ruleCount; x++) {
      ruleList[x].state = ruleList[x+1].state;
      ruleList[x].dataMatch = ruleList[x+1].dataMatch;
      ruleList[x].ruleAction = ruleList[x+1].ruleAction;
      ruleList[x].ruleActionExtra1 = ruleList[x+1].ruleActionExtra1;
      ruleList[x].ruleActionExtra2 = ruleList[x+1].ruleActionExtra2;
      ruleList[x].ruleActionExtra3 = ruleList[x+1].ruleActionExtra3;
      strncpy(ruleList[x].ruleActionExtra4, ruleList[x+1].ruleActionExtra4, 20);
      strncpy(ruleList[x].ruleActionExtra5, ruleList[x+1].ruleActionExtra5, 20);
    }
    ruleCount--;
  }

  // Update states
  for (uint8_t x = 0; x < ruleCount; x++) {
    if (server.arg("state" + (String) x).toInt() == 1) {
      ruleList[x].state = 1;
    }
    else {
      ruleList[x].state = 0;
    }
  }
}


// POST: Save EEPROM
if (server.hasArg("save-eeprom")) {
  temp += "Saving EEPROM";
  
  uint16_t eepromCounter = 0;
  for (uint8_t x = 0; x < ruleCount; x++) {
    if (ruleList[x].dataMatch >= 1) { // 46 bytes per rule
      EEPROM.write(eepromCounter++, ruleList[x].state);
      EEPROM.write(eepromCounter++, ruleList[x].dataMatch);
      EEPROM.write(eepromCounter++, ruleList[x].ruleAction);
      EEPROM.write(eepromCounter++, ruleList[x].ruleActionExtra1);
      EEPROM.write(eepromCounter++, ruleList[x].ruleActionExtra2);
      EEPROM.write(eepromCounter++, ruleList[x].ruleActionExtra3);

      for (uint8_t c = 0; c < 20; c++) {
        EEPROM.write(eepromCounter++, ruleList[x].ruleActionExtra4[c]);
      }
      for (uint8_t c = 0; c < 20; c++) {
        EEPROM.write(eepromCounter++, ruleList[x].ruleActionExtra5[c]);
      }
    }
    else {
      break;
    }
  }
  EEPROM.commit();
}

// Print rules
temp += "<div class='text-css'><p align='center'><b>nRF24 Rules</b></p></div>\
<form method='POST' action=''>\
<div align='center'>\
  <table border='1' width='75%' cellspacing='0' cellpadding='0' bordercolorlight='#000000'>\
    <tr>\
      <td align='center' width='10%'><font class='normaltext-css'><b>No</b></font></td>\
      <td align='center' width='13%'><font class='normaltext-css'><b>State</b></font></td>\
      <td align='center'><font class='normaltext-css'><b>Hex Match</b></font></td>\
      <td align='center' width='29%'><font class='normaltext-css'><b>Rule Action</b></font></td>\
      <td align='center' width='41%'><font class='normaltext-css'><b>Additional Actions</b></font></td>\
      <td align='center' width='5%'><font class='normaltext-css'><b>Delete</b></font></td>\
    </tr>";

for (uint8_t x = 0; x < ruleCount; x++) {
  if (ruleList[x].dataMatch >= 1) {
    temp += "<tr><td align='center' width='10%' height='35'><font class='smalltext-css'>";
    temp += x;  
    temp += "</font></td><td align='center' width='13%' height='35'><input type='checkbox' name='state";
    temp += x;
    temp += "' value='1'";
    
    if (ruleList[x].state == 1) {
      temp += " checked";
    }
   
    temp += "></td><td align='center' height='35'><font class='smalltext-css'>0x";
    
    char dataHex [10];
    sprintf (dataHex, "%X", ruleList[x].dataMatch);
    temp += dataHex;
    
    temp += "</font></td><td align='center' width='29%' height='35'>";
    
    if (ruleList[x].ruleAction == 0) {
      temp += "Send Email";
      temp += "</td><td align='center' width='41%' height='35'><font class='smalltext-css'>";
      temp += "To: ";
      temp += ruleList[x].ruleActionExtra4;
      temp += "<br/>Subject: ";
      temp += ruleList[x].ruleActionExtra5;
      temp += "</font></td><td align='center' width='5%' height='35'><input type='checkbox' name='delete' value='";
      temp += x+1;
      temp += "'></td></tr>";
    }
    else if (ruleList[x].ruleAction == 1) {
      temp += "Pulse Pin";
      temp += "</td><td align='center' width='41%' height='35'><font class='smalltext-css'>";
      temp += "Pin Number: ";
      temp += ruleList[x].ruleActionExtra1;
      temp += "<br/>Pin State: ";
      temp += ruleList[x].ruleActionExtra2;
      temp += "<br/>Pin Time (s): ";
      temp += ruleList[x].ruleActionExtra3;
      temp += "</font></td><td align='center' width='5%' height='35'><input type='checkbox' name='delete' value='";
      temp += x+1;
      temp += "'></td></tr>";
    }
    else if (ruleList[x].ruleAction == 2) {
      temp += "Send NRF24 Packet";
      temp += "</td><td align='center' width='41%' height='35'><font class='smalltext-css'>";
      temp += "nRF CH: ";
      temp += ruleList[x].ruleActionExtra1;
      temp += "<br/>nRF Byte: ";
      temp += ruleList[x].ruleActionExtra2;
      temp += "<br/>nRF TX: ";
      temp += ruleList[x].ruleActionExtra4;
      temp += "<br/>nRF RX: ";
      temp += ruleList[x].ruleActionExtra5;
      
      temp += "</font></td><td align='center' width='5%' height='35'><input type='checkbox' name='delete' value='";
      temp += x+1;
      temp += "'></td></tr>";
    }
    else if (ruleList[x].ruleAction == 3) {
      temp += "Write to variable";
      temp += "</td><td align='center' width='41%' height='35'><font class='smalltext-css'>";
      temp += "Variable No: ";
      temp += ruleList[x].ruleActionExtra1;
      temp += "<br/>Byte: ";
      temp += ruleList[x].ruleActionExtra2;
      temp += "<br/>Timeout(sec): ";
      temp += ruleList[x].ruleActionExtra3;
      
      temp += "</font></td><td align='center' width='5%' height='35'><input type='checkbox' name='delete' value='";
      temp += x+1;
      temp += "'></td></tr>";
    }
    else if (ruleList[x].ruleAction == 4) {
      temp += "Reply with variable";
      temp += "</td><td align='center' width='41%' height='35'><font class='smalltext-css'>";
      temp += "Variable No: ";
      temp += ruleList[x].ruleActionExtra1;
      
      temp += "</font></td><td align='center' width='5%' height='35'><input type='checkbox' name='delete' value='";
      temp += x+1;
      temp += "'></td></tr>";
    }
  }
}

temp += "</tr>\
  </table>\
</div><p align='center'><input type='submit' value='Submit' name='submit-delete'> &nbsp;&nbsp;&nbsp;<input type='submit' value='Save to EEPROM' name='save-eeprom'></p>\
</form>";


// New rules
temp += "<div class='text-css'><p align='center'><b>Add new rule</b></p></div>\
<form method='POST' action=''>\
<div align='center'>\
  <table border='1' width='85%' cellspacing='0' cellpadding='0' bordercolorlight='#000000'>\
    <tr>\
      <td align='center' width='13%'><font class='normaltext-css'><b>State</b></font></td>\
      <td align='center'><font class='normaltext-css'><b>Hex Match</b></font></td>\
      <td align='center' width='29%'><font class='normaltext-css'><b>Rule Action</b></font></td>\
      <td align='center' width='41%'><font class='normaltext-css'><b>Additional Actions</b></font></td>\
    </tr>\
    <tr>\
      <td align='center' width='13%' height='35'>\
      <input type='checkbox' name='state' value='1'></td>\
      <td align='center' height='35'>\
      <input type='text' name='bytematch' size='7'></td>\
      <td align='center' width='29%' height='35'>\
      <select size='1' name='action' id='mySelect' onchange='myFunction()'>\
      <option value='0'>Send Email</option>\
      <option value='1'>Pulse Pin</option>\
      <option value='2'>Send NRF24 Packet</option>\
      <option value='3'>Write to variable</option>\
      <option value='4'>Reply with variable</option>\
      </select></td>\
      <td align='center' width='41%' height='35'><font class='normaltext-css'>\
      <p id='extrafields'>&nbsp;</p></font></td>\
    </tr>\
  </table>\
</div>\
<script>\
function myFunction() {\
    var x = document.getElementById(\"mySelect\").value;\
    if (x == \"0\") {\
      document.getElementById(\"extrafields\").innerHTML = \"<font class='smalltext-css'>To: <input type='text' name='email-to' size='10'><br/>Subject: <input type='text' name='email-subject' size='10'></font>\";\
  }\
    else if (x == \"1\") {\
      document.getElementById(\"extrafields\").innerHTML = \"<font class='smalltext-css'>Pin: <select name='pin-number'><option value='2'>Pin 2<option value='5'>Pin 5<option value='9'>Pin 9<option value='10'>Pin 10<option value='16'>Pin 16</select></select><br/>New State: <select name='pin-state'><option value='1'>High<option value='0'>Low</select><br/>Time(s): <input type='text' name='pin-time' size='10'></font>\";\
  }\
  else if (x == \"2\") {\
      document.getElementById(\"extrafields\").innerHTML = \"<font class='smalltext-css'>nRF TX: <input type='text' name='nrf-tx' size='10'><br/>nRF RX: <input type='text' name='nrf-rx' size='10'><br/>Channel: <input type='text' name='nrf24-ch' size='3'><br/>Packet byte: <input type='text' name='nrf24-byte' size='22'></font>\";\
  }\
  else if (x == \"3\") {\
      document.getElementById(\"extrafields\").innerHTML = \"<font class='smalltext-css'>Variable No: <input type='text' name='variable-number' size='10'><br/>Byte: <input type='text' name='variable-byte' size='10'><br/>Timeout(sec): <input type='text' name='variable-timeout' size='3'></font>\";\
  }\
  else if (x == \"4\") {\
      document.getElementById(\"extrafields\").innerHTML = \"<font class='smalltext-css'>Variable No: <input type='text' name='variable-number' size='10'></font>\";\
  }\
}\
</script>\
<p align='center'><input type='submit' value='Submit' name='Submit'></p>\
</form>";
  
  server.send ( 200, "text/html", temp );
  digitalWrite ( led, 0 );
}

void handleCss() {
  digitalWrite ( led, 1 );
  
  String temp(F(".arm-css {\
  display: inline-block;\
  -webkit-box-sizing: content-box;\
  -moz-box-sizing: content-box;\
  box-sizing: content-box;\
  width: 16%;\
  height: 5%;\
  cursor: pointer;\
  padding: 10px 20px;\
  border: 1px solid #000000;\
  -webkit-border-radius: 3px;\
  border-radius: 3px;\
  font: normal 30px/normal 'Trebuchet MS', Helvetica, sans-serif;\
  color: rgba(255,255,255,0.9);\
  -o-text-overflow: clip;\
  text-overflow: clip;\
  background: rgba(232,126,39,0.66);\
  -webkit-box-shadow: 2px 2px 2px 0 rgba(0,0,0,0.2) ;\
  box-shadow: 2px 2px 2px 0 rgba(0,0,0,0.2) ;\
  -webkit-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  -moz-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  -o-transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
  transition: all 300ms cubic-bezier(0.42, 0, 0.58, 1);\
}\
.text-css {\
  -webkit-box-sizing: content-box;\
  -moz-box-sizing: content-box;\
  box-sizing: content-box;\
  border: none;\
  font: normal 40px/1 'Trebuchet MS', Helvetica, sans-serif;\
  color: black;\
  -o-text-overflow: ellipsis;\
  text-overflow: ellipsis;\
}\
.normaltext-css {\
  -webkit-box-sizing: content-box;\
  -moz-box-sizing: content-box;\
  box-sizing: content-box;\
  border: none;\
  font: normal 30px/1 'Trebuchet MS', Helvetica, sans-serif;\
  color: black;\
  -o-text-overflow: ellipsis;\
  text-overflow: ellipsis;\
}\
.smalltext-css {\
  -webkit-box-sizing: content-box;\
  -moz-box-sizing: content-box;\
  box-sizing: content-box;\
  border: none;\
  font: normal 20px/1 'Trebuchet MS', Helvetica, sans-serif;\
  color: black;\
  -o-text-overflow: ellipsis;\
  text-overflow: ellipsis;\
}"));

  server.send ( 200, "text/html", temp );
  digitalWrite ( led, 0 );
}

void handleNotFound() {
  digitalWrite ( led, 1 );
  
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += ( server.method() == HTTP_GET ) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";

  for ( uint8_t i = 0; i < server.args(); i++ ) {
    message += " " + server.argName ( i ) + ": " + server.arg ( i ) + "\n";
  }

  server.send ( 404, "text/plain", message );
  digitalWrite ( led, 0 );
}


void setup ( void ) {
  pinMode ( led, OUTPUT );
  digitalWrite ( led, 0 );
  Serial.begin ( 115200 );

  WiFi.mode(WIFI_STA);
  WiFi.begin ( ssid, password );
  WiFi.config(ip, gateway, subnet);
  
  // Wait for connection
  while ( WiFi.status() != WL_CONNECTED ) {
    delay ( 500 );
  }

  server.on ( "/", handleRoot );
  server.on ( "/style.css", handleCss );
  server.onNotFound ( handleNotFound );
  server.begin();
  
  mirf_setup();

  // Load rules from EEPROM 
  EEPROM.begin(4096);

  uint16_t eepromCounter = 0;
  for (uint8_t x = 0; x < MAX_RULES; x++) {
    uint8_t dataCheck = EEPROM.read(eepromCounter + 1);
    uint8_t ruleActionCheck = EEPROM.read(eepromCounter + 2);
    /*Serial.println("");
    Serial.print("x ");
    Serial.println(x);
    
    Serial.print("datacheck ");
    Serial.println(dataCheck, HEX);*/
    
    if (dataCheck >= 1 && dataCheck != 255 && ruleActionCheck <= 4) { // 46 bytes per rule
      ruleList[x].state = EEPROM.read(eepromCounter++);
      ruleList[x].dataMatch = EEPROM.read(eepromCounter++);
      ruleList[x].ruleAction = EEPROM.read(eepromCounter++);
      ruleList[x].ruleActionExtra1 = EEPROM.read(eepromCounter++);
      ruleList[x].ruleActionExtra2 = EEPROM.read(eepromCounter++);
      ruleList[x].ruleActionExtra3 = EEPROM.read(eepromCounter++);

      /*Serial.print("state ");
      Serial.println(ruleList[x].state, HEX);
      Serial.print("dataMatch ");
      Serial.println(ruleList[x].dataMatch, HEX);
      Serial.print("ruleAction ");
      Serial.println(ruleList[x].ruleAction, HEX);
      Serial.print("ruleActionExtra1 ");
      Serial.println(ruleList[x].ruleActionExtra1, HEX);
      Serial.print("ruleActionExtra2 ");
      Serial.println(ruleList[x].ruleActionExtra2, HEX);
      Serial.print("ruleActionExtra3 ");
      Serial.println(ruleList[x].ruleActionExtra3, HEX);

      Serial.print("ruleActionExtra4 ");*/
      for (uint8_t c = 0; c < 20; c++) {
        ruleList[x].ruleActionExtra4[c] = EEPROM.read(eepromCounter++);
        //Serial.print(ruleList[x].ruleActionExtra4[c], HEX);
      }
      //Serial.println("");
      //Serial.print("ruleActionExtra5 ");
      for (uint8_t c = 0; c < 20; c++) {
        ruleList[x].ruleActionExtra5[c] = EEPROM.read(eepromCounter++);
        //Serial.print(ruleList[x].ruleActionExtra5[c], HEX);
      }
      //Serial.println("");
      ruleCount++;
    }
    else {
      break;
    }
  }
  
  //Serial.println("\nReady");

  noInterrupts();
  timer0_isr_init();
  timer0_attachInterrupt(timer0_ISR);
  timer0_write(ESP.getCycleCount() + 80000000L); // 80MHz = 1 sec
  interrupts();
}

// Decrease timeout of variables
void timer0_ISR (void) {
  timer0_write(ESP.getCycleCount() + 80000000L); // 80MHz = 1 sec

  // Deincrement common variables timeouts
  for (uint8_t x = 0; x < 10; x++) {
    if (nrfVarsTimeout[x] <= 250) {
      nrfVarsTimeout[x]--;
    }
    else if (nrfVarsTimeout[x] == 0) { // Clear value as timed out
      nrfVars[x] = 0;
    }
  }

  // Check for delay timeouts for pulse pin
  for (uint8_t x = 0; x < MAX_RULES; x++) {
    if (ruleList[x].state == 1 && ruleList[x].ruleAction == 1) {
      if (ruleList[x].ruleActionExtra4[0] >= 1) {
        ruleList[x].ruleActionExtra4[0]--;
        if (ruleList[x].ruleActionExtra4[0] == 0) {
          digitalWrite (ruleList[x].ruleActionExtra1, !ruleList[x].ruleActionExtra2); // Set pin back
        }
      }
    }
  }  
}

void loop ( void ) {
  server.handleClient();

  // Check if we have a packet waiting
  if (mirf_status() & (1<<RX_DR)) {
    mirf_CSN_lo; // Pull down chip select
    SPI.transfer(R_RX_PAYLOAD); // Send cmd to read rx payload
    dataIn = SPI.transfer(0x00); // Read payload
    mirf_CSN_hi; // Pull up chip select
    mirf_config_register(STATUS,(1<<RX_DR)); // Reset status register
    mirf_flush_rx_tx(); // Flush RX and TX FIFO
    
    Serial.print("Packet received: ");
    Serial.println(dataIn, HEX);
    
    // Process packet
    for (uint8_t x = 0; x < MAX_RULES; x++) {
      if (ruleList[x].state == 1 && ruleList[x].dataMatch == dataIn) { // Rule enabled and data matched?
        if (ruleList[x].ruleAction == 0) { // Send email
          if (strlen(ruleList[x].ruleActionExtra4) >= 5) {
            SendEmail e("mail.tpg.com.au", 25, "", "", 6000, false);
            e.send(emailFromName, emailFromAddress, ruleList[x].ruleActionExtra4, ruleList[x].ruleActionExtra5, ruleList[x].ruleActionExtra5);
            Serial.println("Sent email");
          }
        }
        if (ruleList[x].ruleAction == 1) { // Pulse Pin
          digitalWrite (ruleList[x].ruleActionExtra1, ruleList[x].ruleActionExtra2);
          //delay(ruleList[x].ruleActionExtra3 * 1000);
          //digitalWrite (ruleList[x].ruleActionExtra1, !ruleList[x].ruleActionExtra2);
          ruleList[x].ruleActionExtra4[0] = ruleList[x].ruleActionExtra3 + 1; // Let the 1 second timer handle setting the pin back
          Serial.println("Pulsed pin");
        }
        if (ruleList[x].ruleAction == 2) { // Send NRF24 Packet
          mirf_CE_lo; // Stop listening
          mirf_config_register(RF_CH, ruleList[x].ruleActionExtra1); // Change channel
          mirf_write_register(RX_ADDR_P0, (uint8_t*) ruleList[x].ruleActionExtra4, 5); // Change RADDR and TADDR
          mirf_write_register(RX_ADDR_P1, (uint8_t*) ruleList[x].ruleActionExtra5, 5);
          mirf_write_register(TX_ADDR, (uint8_t*) ruleList[x].ruleActionExtra4, 5);
          mirf_transmit_data(ruleList[x].ruleActionExtra2); // Send byte
          delay(5);
          
          mirf_config_register(RF_CH, mirf_CH); // Change channel back
          mirf_write_register(RX_ADDR_P0, TADDR, 5); // Change RADDR and TADDR back
          mirf_write_register(RX_ADDR_P1, RADDR, 5);
          mirf_write_register(TX_ADDR, TADDR, 5);
          
          NRF_RX_POWERUP; // Power up to receiver mode
          delay(3);
          mirf_flush_rx_tx(); // Flush RX and TX FIFO
          mirf_CE_hi; // Start listening

          Serial.println("Sent nRF24 packet");
        }
        if (ruleList[x].ruleAction == 3) { // Write to variable
          uint8_t nrfVarLocation = ruleList[x].ruleActionExtra1;
          nrfVars[nrfVarLocation] = ruleList[x].ruleActionExtra2;
          
          if (ruleList[x].ruleActionExtra3 <= 250) {
            nrfVarsTimeout[nrfVarLocation] = ruleList[x].ruleActionExtra3;
          }
          
          Serial.print("Write to variable location: ");
          Serial.print(nrfVarLocation);
          Serial.print(" with byte: ");
          Serial.println(nrfVars[nrfVarLocation]);
        }
        if (ruleList[x].ruleAction == 4) { // Reply with variable
          mirf_CE_lo; // Stop listening
        
          uint8_t nrfVarLocation = ruleList[x].ruleActionExtra1;
          mirf_transmit_data(nrfVars[nrfVarLocation]); // Send byte
          delay(3);
          
          NRF_RX_POWERUP; // Power up to receiver mode
          delay(3);
          mirf_flush_rx_tx(); // Flush RX and TX FIFO
          mirf_CE_hi; // Start listening

          Serial.print("Reply with variable location: ");
          Serial.print(nrfVarLocation);
          Serial.print(" with byte: ");
          Serial.println(nrfVars[nrfVarLocation]);
        }
      }
    }
  }
}

