/*
    Copyright (c) 2007 Stefan Engelke <mbox@stefanengelke.de>

    Permission is hereby granted, free of charge, to any person 
    obtaining a copy of this software and associated documentation 
    files (the "Software"), to deal in the Software without 
    restriction, including without limitation the rights to use, copy, 
    modify, merge, publish, distribute, sublicense, and/or sell copies 
    of the Software, and to permit persons to whom the Software is 
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be 
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
    DEALINGS IN THE SOFTWARE.

    $Id$
*/

#ifndef uint8_t
 typedef unsigned char uint8_t;
#endif 

#ifndef _MIRF_H_
#define _MIRF_H_

// Mirf settings
#define mirf_CH         11
#define mirf_PAYLOAD    1
#define mirf_CONFIG     ( (1<<EN_CRC) | (0<<CRCO) )
#define RADDR		(byte *)"servr"
#define TADDR		(byte *)"clnt1"

// Pin definitions for chip select and chip enabled of the MiRF module
#define CE  4
#define CSN 15

// Definitions for selecting and enabling MiRF module
#define mirf_CSN_hi     digitalWrite(CSN, HIGH);
#define mirf_CSN_lo     digitalWrite(CSN, LOW);
#define mirf_CE_hi      digitalWrite(CE, HIGH);
#define mirf_CE_lo      digitalWrite(CE, LOW);

// Data in/out elements
#define SENSOR_NUMBER 0
#define LINK_BYTE 1
#define HOURS_DELAY 2

// Public standard functions
extern void mirf_init(void);
extern void mirf_config(void);
extern uint8_t mirf_status(void);
extern uint8_t mirf_data_ready(void);
extern uint8_t mirf_transmit_data(void);
extern uint8_t mirf_receive_data(void);
extern uint8_t process_check_in(void);

// Public extended functions
extern void mirf_config_register(uint8_t reg, uint8_t value);
extern void mirf_read_register(uint8_t reg, uint8_t *value, uint8_t len);
extern void mirf_write_register(uint8_t reg, uint8_t *value, uint8_t len);

// Defines for setting the MiRF registers for transmitting or receiving mode
#define NRF_TX_POWERUP mirf_config_register(CONFIG, mirf_CONFIG | ( (1<<PWR_UP) | (0<<PRIM_RX) ) )
#define NRF_RX_POWERUP mirf_config_register(CONFIG, mirf_CONFIG | ( (1<<PWR_UP) | (1<<PRIM_RX) ) )
#define NRF_POWERDOWN mirf_config_register(CONFIG, mirf_CONFIG | ( (0<<PWR_UP) ) )

#endif /* _MIRF_H_ */

